﻿using PSqLiteWrapper;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SQLite;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;

namespace SQLite_Console
{
	class Program
	{
		static void Main(string[] args)
		{
			string dbFileName = "mydb.db";

			string path = Assembly.GetExecutingAssembly().CodeBase;
			string dir = Path.GetDirectoryName(path).Replace(@"file:\", "");

			Stopwatch watch = new Stopwatch();
			watch.Start();

			PSqLite.DBPath = $@"{dir}\{dbFileName}";
			PSqLite.CreateDB();

			watch.Stop();
			Console.WriteLine($"database check: {watch.ElapsedMilliseconds}ms");
			Console.WriteLine();
			watch.Restart();

			// Create table
			PSqLite.ExecuteQuery(
			@"CREATE TABLE 'member' (
				'name' TEXT,
				'age' INTEGER,
				'guid' TEXT,
				PRIMARY KEY('guid')
			);");

			watch.Stop();
			Console.WriteLine($"create table: {watch.ElapsedMilliseconds}ms");
			Console.WriteLine();
			watch.Restart();

			// Cach size
			PSqLite.ExecuteQuery("PRAGMA cache_size=10000;");

			watch.Stop();
			Console.WriteLine($"set cach size to 10000: {watch.ElapsedMilliseconds}ms");
			Console.WriteLine();
			watch.Restart();

			//List<Thread> threads = new List<Thread>();
			//for (int i = 0; i < 20; i++)
			//{
			//	threads.Add(new Thread(() => InsertRandomRows($"t{i + 1}", 1000)));
			//}

			//foreach (Thread thread in threads)
			//{
			//	thread.Start();
			//}

			Random rnd = new Random();
			for (int i = 0; i < 2000; i++)
			{
				string guid = Guid.NewGuid().ToString();
				int age = rnd.Next(1, 100);
				string name = $"Robo#{i + 1}";
				string insertQry = $"INSERT INTO member VALUES ('{name}', {age}, '{guid}')";
				PSqLite.ExecuteQuery(insertQry);
			}

			watch.Stop();
			Console.WriteLine($"insert 2000 row case1: {watch.ElapsedMilliseconds}ms");
			Console.WriteLine();
			watch.Restart();

			rnd = new Random();
			using (SQLiteConnection conn = PSqLite.Connection)
			{
				conn.Open();
				for (int i = 0; i < 2000; i++)
				{
					string guid = Guid.NewGuid().ToString();
					int age = rnd.Next(1, 100);
					string name = $"Robo#{i + 1}";
					string insertQry = $"INSERT INTO member VALUES ('{name}', {age}, '{guid}')";
					PSqLite.ExecuteQuery(conn, insertQry);
				}

				conn.Close();
			}

			watch.Stop();
			Console.WriteLine($"insert 2000 row case2: {watch.ElapsedMilliseconds}ms");
			Console.WriteLine();
			watch.Restart();

			rnd = new Random();
			string[] queries = new string[2000];
			for (int i = 0; i < 2000; i++)
			{
				string guid = Guid.NewGuid().ToString();
				int age = rnd.Next(1, 100);
				string name = $"Robo#{i + 1}";
				queries[i] = $"INSERT INTO member VALUES ('{name}', {age}, '{guid}')";
			}

			PSqLite.ExecuteQuery(queries);

			watch.Stop();
			Console.WriteLine($"insert 2000 row case3: {watch.ElapsedMilliseconds}ms");
			Console.WriteLine();
			watch.Restart();

			//DataSet select = PSqLite.ExecuteSelectQuery("SELECT * FROM member");
			//DataRowCollection rows = select.Tables[0].Rows;
			////PrintRows(rows);

			//watch.Stop();
			//Console.WriteLine($"select all: {watch.ElapsedMilliseconds}ms");
			//Console.WriteLine();
			//watch.Restart();

			//PSqLite.ExecuteQuery("UPDATE member SET name = 'minor' WHERE age <= 20");

			//watch.Stop();
			//Console.WriteLine($"update under 20: {watch.ElapsedMilliseconds}ms");
			//Console.WriteLine();

			//select = PSqLite.ExecuteSelectQuery("SELECT * FROM member WHERE age <= 20");
			//rows = select.Tables[0].Rows;

			//watch.Stop();
			//Console.WriteLine($"select under 20: {watch.ElapsedMilliseconds}ms");
			//Console.WriteLine();
			//watch.Restart();

			//PrintRows(rows);

			Console.ReadKey();
		}

		private static void PrintRows(DataRowCollection rows)
		{
			Console.WriteLine($"found {rows.Count} rows.");
			foreach (DataRow row in rows)
			{
				string name = (string)row["name"];
				long age = DBNull.Value.Equals(row["age"]) ? -1 : (long)row["age"];
				string guid = (string)row["guid"];

				Console.WriteLine($"name: {name}, age: {age}, guid: {guid}");
			}
		}

		private static void InsertRandomRows(string identifier, int count)
		{
			Console.WriteLine(identifier);

			Stopwatch watch = new Stopwatch();
			watch.Start();

			Random rnd = new Random();
			using (SQLiteConnection conn = PSqLite.Connection)
			{
				conn.Open();
				PSqLite.ExecuteQuery("BEGIN TRANSACTION;");

				for (int i = 0; i < count; i++)
				{
					string guid = Guid.NewGuid().ToString();
					int age = rnd.Next(1, 100);
					string name = $"{identifier} Robo#{i + 1}";
					string insertQry = $"INSERT INTO member VALUES ('{name}', {age}, '{guid}');";
					PSqLite.ExecuteQuery(conn, insertQry);
				}

				PSqLite.ExecuteQuery("BEGIN TRANSACTION;");
				conn.Close();
			}

			watch.Stop();
			Console.WriteLine($"insert 2000 row for {identifier}: {watch.ElapsedMilliseconds}ms");
			Console.WriteLine();
		}
	}
}
